#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""shaper manager - manage library"""

import fnmatch
import os

from . import libs


def walk_on_path(path):
    """Recursively find files with pattern."""

    for root, _, files in os.walk(path):
        for pattern in libs.PARSERS_MAPPING:
            for filename in fnmatch.filter(files, '*{ext}'.format(ext=pattern)):
                yield os.path.join(root, filename)


def create_folders(path_to_folder):
    """Recursively creating folders."""

    try:
        os.makedirs(path_to_folder)
    except OSError:
        if not os.path.isdir(path_to_folder):
            raise EOFError


def read_properties(_dir):
    """Interface for reading properties recursively."""

    result = {
        filename: libs.parser.read(filename) for filename in walk_on_path(_dir)
    }

    return {key: value for key, value in result.items() if value}


def write_properties(datastructure, path):
    """Interface for writing properties recursively."""

    for filename, properties in datastructure.items():
        directories = os.path.join(
            path,
            os.path.dirname(filename)
        )
        create_folders(directories)

        property_file = os.path.basename(filename)
        libs.parser.write(
            properties,
            os.path.join(directories, property_file),
        )


def forward_path_parser(_input):
    """Parsing plain dict to nested."""

    def create_keys_recursively(key, current_tree):
        """Update current tree by key(s)."""

        if key not in current_tree:
            last = keys.pop()
            # pylint: disable=undefined-loop-variable
            # this value defined in the shared outer-function scope
            dict_update = {last: value}

            for _key in reversed(keys):
                dict_update = {_key: dict_update}

            current_tree.update(dict_update)
        else:
            keys.pop(0)  # drop the first item that already in the tree, try next
            create_keys_recursively(keys[0], current_tree[key])

    output = {}
    for key, value in _input.items():
        keys = key.split('/')

        create_keys_recursively(keys[0], output)

    return output


def backward_path_parser(_input):
    """Make nested structure plain."""

    def path_builder(current_tree, key=''):
        """Join all the keys from tree into right path."""

        for _key, _value in current_tree.items():
            _key = key + '/' + _key if key else _key
            if '.' in _key:
                output.update({_key: _value})
            else:
                path_builder(_value, _key)

    output = {}
    path_builder(_input)

    return output



#!/usr/bin/env python
# coding: utf-8

"""
Converts a Python dictionary or other native data type into a valid XML string.

Supports item (`int`, `float`, `long`, `decimal.Decimal`, `bool`, `str`,
`unicode`, `datetime`, `none` and other number-like objects) and collection
(`list`, `set`, `tuple` and `dict`, as well as iterable and dict-like objects)
data types, with arbitrary nesting for the collections. Items with a `datetime`
 type are converted to ISO format strings. Items with a `None` type become
 empty XML elements.
"""

import collections
import logging
import numbers
from random import randint
from xml.dom.minidom import parseString

try:
    unicode, long
except NameError:
    unicode = str  # pylint: disable=redefined-builtin
    long = int  # pylint: disable=redefined-builtin

logger = logging.getLogger("dicttoxml")


def set_debug(debug=True, filename='dicttoxml.log'):
    if debug:
        import datetime
        logging.basicConfig(filename=filename, level=logging.INFO)
        logger.info('\nLogging session starts: %s', datetime.datetime.today())
    else:
        logging.basicConfig(level=logging.WARNING)


def unicode_me(something):
    """Converts strings with non-ASCII characters to unicode for logger.
    Python 3 doesn't have a `unicode()` function, so `unicode()` is an alias
    for `str()`, but `str()` doesn't take a second argument, hence this kludge.
    """
    try:
        return unicode(something, 'utf-8')
    except TypeError:
        return unicode(something)


def make_id(element, start=100000, end=999999):
    """Returns a random integer"""
    return '%s_%s' % (element, randint(start, end))


def get_unique_id(element):
    """Returns a unique id for a given element"""
    ids = []
    this_id = make_id(element)
    dup = True
    while dup:
        if this_id not in ids:
            dup = False
            ids.append(this_id)
        else:
            this_id = make_id(element)
    return ids[-1]


def get_xml_type(val):
    """Returns the data type for the xml type attribute"""
    if isinstance(val, (str, unicode)):
        return 'str'
    if isinstance(val, (int, long)):
        return 'int'
    if isinstance(val, float):
        return 'float'
    if isinstance(val, bool):
        return 'bool'
    if isinstance(val, numbers.Number):
        return 'number'
    if val is None:
        return 'null'
    if isinstance(val, dict):
        return 'dict'
    if isinstance(val, collections.Iterable):
        return 'list'

    return type(val).__name__


def escape_xml(s):
    if isinstance(s, (str, unicode)):
        s = unicode_me(s)  # avoid UnicodeDecodeError
        s = s.replace('&', '&amp;')
        s = s.replace('"', '&quot;')
        s = s.replace('\'', '&apos;')
        s = s.replace('<', '&lt;')
        s = s.replace('>', '&gt;')

    return s


def make_attr_string(attr):
    """Returns an attribute string in the form key="val"."""
    attr_string = ' '.join('%s="%s"' % (k, v) for k, v in attr.items())
    return '%s%s' % (' ' if attr_string != '' else '', attr_string)


def key_is_valid_xml(key):
    """Checks that a key is a valid XML name"""
    logger.info('Inside key_is_valid_xml(). Testing "%s"', unicode_me(key))
    test_xml = '<?xml version="1.0" encoding="UTF-8" ?><%s>foo</%s>' % (key, key)
    try:
        parseString(test_xml)
        return True
    except Exception:  # pylint: disable=broad-except
        return False


def make_valid_xml_name(key, attr):
    """Tests an XML name and fixes it if invalid"""
    logger.info(
        'Inside make_valid_xml_name(). Testing key "%s" with attr "%s"',
        unicode_me(key), unicode_me(attr),
    )
    key = escape_xml(key)
    attr = escape_xml(attr)

    # pass through if key is already valid
    if key_is_valid_xml(key):
        return key, attr

    # prepend a lowercase n if the key is numeric
    if key.isdigit():
        return 'n%s' % (key), attr

    # replace spaces with underscores if that fixes the problem
    if key_is_valid_xml(key.replace(' ', '_')):
        return key.replace(' ', '_'), attr

    # key is still invalid - move it into a name attribute
    attr['name'] = key
    key = 'key'
    return key, attr


def wrap_cdata(s):
    """Wraps a string into CDATA sections"""
    s = unicode_me(s).replace(']]>', ']]]]><![CDATA[>')
    return '<![CDATA[' + s + ']]>'


def convert(obj, ids, attr_type, item_func, cdata, fold_list, parent='root'):
    """Routes the elements of an object to the right function to convert them
    based on their data type"""

    logger.info(
        'Inside convert(). obj type is: "%s", obj="%s"',
        type(obj).__name__, unicode_me(obj),
    )

    item_name = item_func(parent)

    if isinstance(obj, (numbers.Number, str, unicode)):
        return convert_kv(item_name, obj, attr_type, cdata)

    if hasattr(obj, 'isoformat'):
        return convert_kv(item_name, obj.isoformat(), attr_type, cdata)

    if isinstance(obj, bool):
        return convert_bool(item_name, obj, attr_type)

    if obj is None:
        return convert_none(item_name, '', attr_type)

    if isinstance(obj, dict):
        return convert_dict(
            obj, ids, parent, attr_type, item_func, cdata, fold_list)

    if isinstance(obj, collections.Iterable):
        return convert_list(
            obj, ids, parent, attr_type, item_func, cdata, fold_list)

    raise TypeError(
        'Unsupported data type: %s (%s)' % (obj, type(obj).__name__))


def convert_dict(obj, ids, parent, attr_type, item_func, cdata, fold_list):
    """Converts a dict into an XML string."""
    logger.info(
        'Inside convert_dict(): obj type is: "%s", obj="%s"',
        type(obj).__name__, unicode_me(obj),
    )

    output = []
    for key, val in obj.items():
        logger.info(
            'Looping inside convert_dict(): key="%s", val="%s", type(val)="%s"',
            unicode_me(key), unicode_me(val), type(val).__name__,
        )

        attr = {} if not ids else {'id': get_unique_id(parent)}

        key, attr = make_valid_xml_name(key, attr)

        if isinstance(val, (numbers.Number, str, unicode)):
            output.append(convert_kv(key, val, attr_type, cdata, **attr))

        elif hasattr(val, 'isoformat'):  # datetime
            output.append(
                convert_kv(key, val.isoformat(), attr_type, cdata, **attr),
            )

        elif isinstance(val, bool):
            output.append(convert_bool(key, val, attr_type, **attr))

        elif isinstance(val, dict):
            if attr_type:
                attr['type'] = get_xml_type(val)

            output.append(
                '<%s%s>%s</%s>' % (
                    key,
                    make_attr_string(attr),
                    convert_dict(val, ids, key, attr_type, item_func, cdata, fold_list),
                    key,
                ),
            )

        elif isinstance(val, collections.Iterable):
            if attr_type:
                attr['type'] = get_xml_type(val)

            if fold_list:
                output.append(
                    '<%s%s>%s</%s>' % (
                        key,
                        make_attr_string(attr),
                        convert_list(val, ids, key, attr_type, item_func, cdata, fold_list),
                        key,
                    ),
                )
            else:
                output.append(
                    convert_list(val, ids, key, attr_type, item_func, cdata, fold_list),
                )

        elif val is None:
            output.append(convert_none(key, val, attr_type, **attr))

        else:
            raise TypeError(
                'Unsupported data type: %s (%s)' % (val, type(val).__name__),
            )

    return ''.join(output)


def convert_list(items, ids, parent, attr_type, item_func, cdata, fold_list):
    """Converts a list into an XML string."""
    logger.info('Inside convert_list()')

    if fold_list:
        item_name = item_func(parent)
    else:
        item_name = parent

    output = []
    for i, item in enumerate(items):
        logger.info(
            'Looping inside convert_list(): item="%s", item_name="%s", type="%s"',
            unicode_me(item), item_name, type(item).__name__,
        )
        attr = {} if not ids else {'id': '%s_%s' % (get_unique_id(parent), i + 1)}

        if isinstance(item, (numbers.Number, str, unicode)):
            output.append(convert_kv(item_name, item, attr_type, cdata, **attr))

        elif hasattr(item, 'isoformat'):  # datetime
            output.append(
                convert_kv(item_name, item.isoformat(), attr_type, cdata, **attr),
            )

        elif isinstance(item, bool):
            output.append(convert_bool(item_name, item, attr_type, **attr))

        elif isinstance(item, dict):
            if not attr_type:
                output.append(
                    '<%s>%s</%s>' % (
                        item_name,
                        convert_dict(item, ids, parent, attr_type, item_func, cdata, fold_list),
                        item_name,
                    ),
                )
            else:
                output.append(
                    '<%s type="dict">%s</%s>' % (
                        item_name,
                        convert_dict(item, ids, parent, attr_type, item_func, cdata, fold_list),
                        item_name,
                    ),
                )

        elif isinstance(item, collections.Iterable):
            if not attr_type:
                output.append(
                    '<%s%s>%s</%s>' % (
                        item_name,
                        make_attr_string(attr),
                        convert_list(item, ids, item_name, attr_type, item_func, cdata, fold_list),
                        item_name,
                    ),
                )
            else:
                output.append(
                    '<%s type="list"%s>%s</%s>' % (
                        item_name, make_attr_string(attr),
                        convert_list(item, ids, item_name, attr_type, item_func, cdata, fold_list),
                        item_name,
                    ),
                )

        elif item is None:
            output.append(convert_none(item_name, None, attr_type, **attr))

        else:
            raise TypeError(
                'Unsupported data type: %s (%s)' % (item, type(item).__name__),
            )

    return ''.join(output)


def convert_kv(key, val, attr_type, cdata, **attr):
    """Converts a number or string into an XML element"""
    logger.info(
        'Inside convert_kv(): key="%s", val="%s", type(val) is: "%s"',
        unicode_me(key), unicode_me(val), type(val).__name__,
    )

    key, attr = make_valid_xml_name(key, attr)

    if attr_type:
        attr['type'] = get_xml_type(val)

    attr_string = make_attr_string(attr)

    return '<%s%s>%s</%s>' % (
        key,
        attr_string,
        wrap_cdata(val) if cdata else escape_xml(val),
        key,
    )


def convert_bool(key, val, attr_type, **attr):
    """Converts a boolean into an XML element"""
    logger.info(
        'Inside convert_bool(): key="%s", val="%s", type(val) is: "%s"',
        unicode_me(key), unicode_me(val), type(val).__name__,
    )

    key, attr = make_valid_xml_name(key, attr)

    if attr_type:
        attr['type'] = get_xml_type(val)

    attr_string = make_attr_string(attr)

    return '<%s%s>%s</%s>' % (key, attr_string, unicode(val).lower(), key)


def convert_none(key, val, attr_type, **attr):
    """Converts a null value into an XML element"""
    logger.info('Inside convert_none(): key="%s"', unicode_me(key))

    key, attr = make_valid_xml_name(key, attr)

    if attr_type:
        attr['type'] = get_xml_type(val)

    attr_string = make_attr_string(attr)

    return '<%s%s></%s>' % (key, attr_string, key)


def dict_to_xml(
        obj, root=True, custom_root='root', ids=False, attr_type=True,
        item_func=lambda x: 'item', cdata=False, fold_list=True,
):
    """Converts a python object into XML.
    Arguments:
    - root specifies whether the output is wrapped in an XML root element
      Default is True
    - custom_root allows you to specify a custom root element.
      Default is 'root'
    - ids specifies whether elements get unique ids.
      Default is False
    - attr_type specifies whether elements get a data type attribute.
      Default is True
    - item_func specifies what function should generate the element name for
      items in a list.
      Default is 'item'
    - cdata specifies whether string values should be wrapped in CDATA sections.
      Default is False
    """
    logger.info(
        'Inside dict_to_xml(): type(obj) is: "%s", obj="%s"',
        type(obj).__name__, unicode_me(obj),
    )

    output = []
    if root:
        output.append('<?xml version="1.0" encoding="UTF-8" ?>')
        output.append(
            '<%s>%s</%s>' % (
                custom_root,
                convert(obj, ids, attr_type, item_func, cdata, fold_list, parent=custom_root),
                custom_root,
            ),
        )
    else:
        output.append(
            convert(obj, ids, attr_type, item_func, cdata, fold_list, parent=''),
        )

    return ''.join(output).encode('utf-8')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Shaper (CMDB tool)
    Lib for manipulating files: read/write

    Support ivan.bogomazov@gmail.com
    Minsk 2018


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    EXAMPLES:
        TBD
"""

import json
import os
import sys
from collections import OrderedDict
from xml.dom.minidom import parseString

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import xmltodict
import yaml

from . import dicttoxml
from .loader import (
    OrderedDictYAMLLoader,
    represent_ordered_dict,
    represent_unicode,
    represent_multi_line,
)


class BaseParser(object):
    WARNING_MESSAGE = 'Warning. Unsupported file extension for {file}'

    @staticmethod
    def parsers_choice(filepath):
        """Get parser class by file type.

        :param filepath: string path to file
        :return: parser class
        """

        _, ext = os.path.splitext(filepath)

        return PARSERS_MAPPING.get(ext)

    def read(self, path):  # pylint: disable=inconsistent-return-statements
        """Read file data structure according its type. Default type choose
        dynamic with magic function.

        :param path: string path to file
        :return: File data structure
        :rtype: [dict, list]
        """

        parser_class = self.parsers_choice(path)
        if parser_class:
            try:
                return parser_class().read(path)

            # pylint: disable=broad-except
            # disable cause of list of exceptions
            # not known due to a lot of parsers
            except Exception as exc:
                msg = 'Failed to parse {file}'.format(file=os.path.abspath(path))
                sys.stderr.write(
                    '{message}\n{exception}'.format(message=msg, exception=exc),
                )

        sys.stderr.write(self.WARNING_MESSAGE.format(file=path))

    def write(self, data, path):
        """Write data in file according its type. Default type choose dynamic
        with magic function.

        :param path: string path to file
        :param data: data
        :type data: [str, dict, list]

        :return: None
        :rtype: None
        """

        parser_class = self.parsers_choice(path)
        if parser_class:
            parser_class().write(data, path)
        else:
            sys.stderr.write(self.WARNING_MESSAGE.format(file=path))


class TextParser(object):

    def read(self, path):
        """Read plaintext file.

        :param path: string path to file
        :return: list of lines of string
        :rtype: [list, str]
        """

        try:
            with open(path, 'r') as fd:
                return fd.read()

        except (ValueError, OSError, IOError) as exc:
            sys.stderr.write(
                'Failed to read {file}: {msg}'.format(file=path, msg=str(exc)),
            )

    def write(self, data, path):
        """Write plaintext file.

        :param data: file content
        :param path: string path to file
        :return: str

        :return: None
        :rtype: None
        """

        try:
            with open(path, 'wb') as fd:
                fd.write(data)

        except (ValueError, OSError, IOError) as exc:
            sys.stderr.write(
                'Failed to write {file}: {msg}'.format(file=path, msg=str(exc)),
            )


class YAMLParser(TextParser):

    def read(self, path):
        """YAML read.

        :param path: string path to file
        :return: data structure
        :rtype: dict
        """

        return yaml.load(
            super(YAMLParser, self).read(path),
            Loader=OrderedDictYAMLLoader,
        )

    def write(self, data, path):
        """Dump data structure to YAML.

        :param data: configuration dataset
        :param path: string path to file
        :type data: dict

        :return: None
        :rtype: None
        """

        yaml.add_representer(OrderedDict, represent_ordered_dict)
        if sys.version_info[0] == 2:
            yaml.add_representer(
                unicode,  # pylint: disable=undefined-variable
                represent_unicode,
            )

        # add string representer for multi line issue
        yaml.add_representer(str, represent_multi_line)

        content = yaml.dump(
            data,
            default_flow_style=False,
            allow_unicode=True,
        )

        if sys.version_info[0] == 3:
            content = bytearray(content, 'utf-8')

        super(YAMLParser, self).write(content, path)


class JSONParser(TextParser):

    def read(self, path):
        """JSON read.

        :param path: string path to file
        :return: json data structure
        :rtype: dict
        """

        return json.loads(super(JSONParser, self).read(path))

    def write(self, data, path):
        """Dump data to JSON.

        :param data: configuration data structure
        :param path: string path to file
        :type data: dict

        :return: None
        :rtype: None
        """

        kw = {'encoding': 'utf-8'} if sys.version_info[0] == 2 else {}
        with open(path, 'w') as fd:
            json.dump(
                data,
                fd,
                indent=4,
                separators=(',', ': '),
                **kw
            )


class XMLParser(TextParser):

    def read(self, path):
        """XML read.

        :param path: string path to file
        :return: XML data structure
        :rtype: dict
        """

        return xmltodict.parse(super(XMLParser, self).read(path))

    def write(self, data, path):
        """Dump data structure to XML.

        :param data: configuration data structure
        :param path: string path to file
        :type data: dict

        :return: None
        :rtype: None
        """

        dom = parseString(
            dicttoxml.dict_to_xml(
                data,
                fold_list=False,
                item_func=lambda x: x,
                attr_type=False,
                root=False,
            ),
        )

        super(XMLParser, self).write(dom.toprettyxml(encoding='utf-8'), path)


class PropertyParser(TextParser):

    @staticmethod
    def _process_multiline_string(string):
        string_splitted = string.splitlines()
        if len(string_splitted) > 1:
            return "\n  ".join(string_splitted)
        return string

    def read(self, path):
        """PROPERTY read.

        :param path: string path to file
        :return: property data structure
        :rtype: dict
        """

        try:
            import ConfigParser
        except ImportError:
            import configparser as ConfigParser

        content = super(PropertyParser, self).read(path)

        config = StringIO()
        config.write('[dummy_section]\n')
        config.write(content.replace('%', '%%'))
        config.seek(0, os.SEEK_SET)

        conf_parser = ConfigParser.SafeConfigParser()
        conf_parser.optionxform = str
        # pylint: disable=deprecated-method
        conf_parser.readfp(config)

        return OrderedDict(conf_parser.items('dummy_section'))

    def write(self, data, path):
        """Dump data structure to property.

        :param data: configuration data structure
        :param path: string path to file
        :type data: dict

        :return: None
        :rtype: None
        """

        stream = '\n'.join(
            '{}={}'.format(item[0], self._process_multiline_string(item[1])) for item in data.items(),
        )
        super(PropertyParser, self).write(
            stream.encode(encoding='utf-8'),
            path,
        )


parser = BaseParser()

PARSERS_MAPPING = {
    '.json': JSONParser,
    '.yml': YAMLParser,
    '.yaml': YAMLParser,
    '.xml': XMLParser,
    '.properties': PropertyParser,
    '.txt': TextParser,
    # '': TextParser, TODO: think about how to parse files without extension
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Shaper (CMDB tool)
    Parse properties&configs to datastructure
    and create properties&configs from datastructure.

    Support ivan.bogomazov@gmail.com
    Minsk 2018
"""

from . import libs

from ._version import get_versions

__version__ = get_versions()['version']


__all__ = ['libs', '__version__']



import os
import shutil
from collections import OrderedDict

from shaper import manager, libs


def test_create_folder():
    temp_dir_name = 'test_folder'
    manager.create_folders(temp_dir_name)
    assert os.path.isdir(temp_dir_name)
    shutil.rmtree(temp_dir_name)


def test_read_properties(test_assets_root):
    input_dir = test_assets_root / 'input'
    filename_data_map = manager.read_properties(input_dir)

    for filename, data in filename_data_map.items():
        assert os.path.splitext(filename)[1] in libs.PARSERS_MAPPING
        assert data is not None
        assert isinstance(data, (dict, OrderedDict))


def test_forward_path_parser():
    datastructure = {
        'g/e/c6': 'c6',
        'a/d/c3': 'c3',
        'a/b/c1': 'c1',
        'a/c4': 'c4',
        'a/b/c2': 'c2',
        'g/c5': 'c5',
    }

    expected = {
        'a': {
            'b': {'c1': 'c1', 'c2': 'c2'},
            'c4': 'c4',
            'd': {'c3': 'c3'},
        },
        'g': {
            'c5': 'c5',
            'e': {'c6': 'c6'},
        },
    }

    assert expected == manager.forward_path_parser(datastructure)


def test_backward_path_parser():
    datastructure = OrderedDict(
        [
            (
                'a',
                OrderedDict(
                    [
                        ('c4.py', 'c4'),
                        ('d', OrderedDict([('c3.py', 'c3')])),
                        ('b', OrderedDict([('c2.py', 'c2'), ('c1.py', 'c1')])),
                    ],
                ),
            ),
            ('g', OrderedDict([('c5.py', 'c5'), ('e', OrderedDict([('c6.py', 'c6')]))])),
        ],
    )

    expected = {
        'a/b/c1.py': 'c1',
        'a/b/c2.py': 'c2',
        'a/c4.py': 'c4',
        'a/d/c3.py': 'c3',
        'g/c5.py': 'c5',
        'g/e/c6.py': 'c6',
    }

    assert expected == manager.backward_path_parser(datastructure)




import os
import shutil

from shaper import manager, libs


def test_read(test_assets_root):
    """
    This test is sensitive to the input directory relative path value.
    Run it only from the project root dir.
    """
    input_dir = test_assets_root / 'input'
    output_file_path = test_assets_root / 'test_output.yml'
    expected_file_path = test_assets_root / 'expected_out.yml'

    gathered_data = manager.read_properties(input_dir.relpath())
    tree = manager.forward_path_parser(gathered_data)

    libs.parser.write(tree, output_file_path)

    output_data = libs.parser.read(output_file_path)
    output_data = manager.backward_path_parser(output_data)

    expected_data = libs.parser.read(expected_file_path)
    expected_data = manager.backward_path_parser(expected_data)

    for key, value in output_data.items():
        assert expected_data[key] == value

    os.remove(output_file_path)


def test_write(test_assets_root):
    output_dir = test_assets_root / 'output'
    input_file_path = test_assets_root / 'expected_out.yml'

    data = libs.parser.read(input_file_path)
    datastructure = manager.backward_path_parser(data)

    manager.write_properties(datastructure, output_dir)

    output_dir_listing = manager.walk_on_path(output_dir)
    expected_dir_listing = manager.walk_on_path(test_assets_root / 'input')
    for output_file, expected_file in zip(output_dir_listing, expected_dir_listing):
        output_data = libs.parser.read(output_file)
        expected_data = libs.parser.read(expected_file)

        assert output_data == expected_data

    shutil.rmtree(output_dir)


def test_play():
    pass



from collections import OrderedDict

import yaml
from yaml.constructor import ConstructorError


class OrderedDictYAMLLoader(yaml.Loader):  # pylint: disable=too-many-ancestors
    """
    A YAML loader that loads mappings into ordered dictionaries.
    """

    def __init__(self, *args, **kwargs):
        yaml.Loader.__init__(self, *args, **kwargs)

        OrderedDictYAMLLoader.add_constructor(
            u'tag:yaml.org,2002:map',
            OrderedDictYAMLLoader.construct_yaml_map,
        )
        OrderedDictYAMLLoader.add_constructor(
            u'tag:yaml.org,2002:omap',
            OrderedDictYAMLLoader.construct_yaml_map,
        )

    def construct_yaml_map(self, node):
        data = OrderedDict()
        yield data
        value = self.construct_mapping(node)
        data.update(value)

    def construct_mapping(self, node, deep=False):
        if isinstance(node, yaml.MappingNode):
            self.flatten_mapping(node)
        else:
            raise ConstructorError(
                None,
                None,
                'expected a mapping node, but found %s' % node.id,
                node.start_mark,
            )

        mapping = OrderedDict()
        for key_node, value_node in node.value:
            key = self.construct_object(key_node, deep=deep)
            try:
                hash(key)
            except TypeError as exc:
                raise ConstructorError(
                    'while constructing a mapping',
                    node.start_mark,
                    'found unacceptable key (%s)' % exc, key_node.start_mark,
                )
            value = self.construct_object(value_node, deep=deep)
            mapping[key] = value
        return mapping


def represent_ordered_dict(dumper, data):
    """Function for Ordered Dictionary representation."""

    value = []

    for item_key, item_value in data.items():
        node_key = dumper.represent_data(item_key)
        node_value = dumper.represent_data(item_value)

        value.append((node_key, node_value))

    return yaml.nodes.MappingNode(u'tag:yaml.org,2002:map', value)


def represent_unicode(dumper, _unicode):  # pylint: disable=unused-argument
    """Function for unicode representation."""

    return yaml.ScalarNode(u'tag:yaml.org,2002:str', _unicode)


# string representer for multi line issue
def represent_multi_line(dumper, data):
    style = None
    if len(data.splitlines()) > 1:  # check for multi line string
        style = '|'
    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style=style)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""shaper renderer - tools to working with templates"""
from __future__ import print_function

import os
import yaml

from jinja2 import Environment, FileSystemLoader, Undefined


class IgnoreUndefinedAttr(Undefined):  # pylint: disable=too-few-public-methods
    """
    Class for ignoring undefined attributes
    """
    def __getattr__(self, name):
        return None


def render_template(template_path, context):
    """
    Render template interface

    :param template_path: path to template
    :type template_path: str

    :param context: variables
    :type context: dict

    :return: rendered template
    :rtype: str
    """

    env = Environment(
        loader=FileSystemLoader(os.path.dirname(template_path)),
        undefined=IgnoreUndefinedAttr,
    )
    env.globals.update(context)
    template = env.get_template(os.path.basename(template_path))
    return template.render()


def merge_templates(rendered_templates, template_dir):
    """
    Merge templates

    :param rendered_templates: list of rendered templates to merge

    :param template_dir: path to rendered templates

    :return: None
    """
    dict_base = {}
    for var in rendered_templates:
        dict_base.update(yaml.safe_load(var))

    with open(os.path.join(template_dir, 'templates.yaml'), 'w') as _fd:
        yaml.dump(dict_base, _fd, default_flow_style=False)



import os
import sys
from setuptools import find_packages, setup
import versioneer


PROJECT_DIR = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))

# allow setup.py to be run from any path
os.chdir(PROJECT_DIR)


with open(os.path.join(PROJECT_DIR, 'README.md')) as readme:
    description = readme.read()


with open(os.path.join(PROJECT_DIR, 'requirements/base')) as req_fd:
    install_requires = req_fd.readlines()

if (sys.version_info > (3, 0)):
    with open(os.path.join(PROJECT_DIR, 'requirements/python3')) as req_fd:
        install_requires += req_fd.readlines()


with open(os.path.join(PROJECT_DIR, 'requirements/test')) as req_fd:
    test_requires = req_fd.readlines()


with open(os.path.join(PROJECT_DIR, 'requirements/debug')) as req_fd:
    debug_requires = req_fd.readlines()


dev_requires = test_requires + debug_requires


setup(
    name='shaper',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=find_packages(),
    include_package_data=True,
    description='Manage application options',
    long_description=description,
    url='https://github.com/arno49/shaper/wiki',
    author='Ivan Bogomazov',
    author_email='ivan.bogomazov@gmail.com',
    install_requires=install_requires,
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Topic :: Utilities',
        'License :: OSI Approved :: Apache Software License',
    ],
    entry_points={
        'console_scripts': [
            'shaper = shaper.cli:main'
        ]
    },
    extras_require={
        'test': test_requires,
        'dev': dev_requires
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Shaper (CMDB tool)
    Parse properties&configs to datastructure
    and create properties&configs from datastructure.

    Support ivan.bogomazov@gmail.com
    Minsk 2018


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    EXAMPLES:
        TBD
"""

from __future__ import print_function

import argparse
import os

from collections import OrderedDict

from shaper import libs
from shaper import manager
from shaper.renderer import render_template
from shaper.renderer import merge_templates


def parse_arguments():
    parser = argparse.ArgumentParser(
        description='Tool to manage java properties',
    )

    parser.add_argument(
        '-v',
        dest='logging',
        action='count',
        default=0,
        help='Verbose output',
    )

    subparsers = parser.add_subparsers(
        dest='parser',
    )

    read = subparsers.add_parser(
        'read',
        help='Recursive read properties from files.',
    )

    write = subparsers.add_parser(
        'write',
        help='Write properties files from datastructure.',
    )

    play = subparsers.add_parser(
        'play',
        help='Run playbook like ansible.',
    )

    read.add_argument(
        'src_path',
        type=str,
        help='Path to properties directory.',
    )

    read.add_argument(
        '-o',
        '--out',
        dest='out',
        default='out.yml',
        help='Output file. Default out.yaml.',
    )

    write.add_argument(
        'src_structure',
        type=str,
        help='Path to yaml with datastructure.',
    )

    write.add_argument(
        '-o',
        '--out',
        dest='out',
        default='./out/',
        help='Path to output directory. Default ./out/.',
    )

    write.add_argument(
        '-k',
        '--key',
        dest='key',
        default=None,
        help='Key for rendering custom subtree. Default render from root.',
    )

    play.add_argument(
        'src_path',
        type=str,
        help='Path to playbook.',
    )

    return parser.parse_args()


def main():
    arguments = parse_arguments()

    if arguments.parser == 'play':
        playbook = libs.parser.read(arguments.src_path)
        context = playbook.get('variables', {})
        templates = playbook.get('templates', [])
        template_dir = os.path.dirname(arguments.src_path)

        rendered_templates = [
            render_template(template, context) for template in templates
        ]

        merge_templates(rendered_templates, template_dir)

    elif arguments.parser == 'read':
        gathered_data = manager.read_properties(arguments.src_path)
        tree = manager.forward_path_parser(gathered_data)

        libs.parser.write(tree, arguments.out)

    elif arguments.parser == 'write':
        dict_data = libs.parser.read(arguments.src_structure)
        datastructure = manager.backward_path_parser(dict_data)

        # filter render files by key
        if arguments.key:
            datastructure = OrderedDict(
                (key, value)
                for key, value in datastructure.items() if arguments.key in key
            )

        if arguments.logging:
            print('==> Files to render :')
            print('\n'.join(datastructure.keys()))

        manager.write_properties(datastructure, arguments.out)

    else:
        raise NotImplementedError


if __name__ == '__main__':
    main()



from path import Path


def find_git_root(folder):
    folder = Path(folder).abspath()
    while True:
        git_config = folder / '.git' / 'config'
        if git_config.exists():
            return folder
        folder = folder.parent
        if folder == '/':
            raise RuntimeError('Cannot find git repo')
